<?php

namespace App\Livewire;

use Livewire\Component;
use Livewire\WithPagination;
use App\Models\Predio;
use Illuminate\Support\Facades\Validator;

class Predios extends Component
{
    use WithPagination;

    public $search = ''; // Variable de búsqueda
    public $perPage = 10; // Número de elementos por página
    public $modalVisible = false;
    public $modoEdicion = false;
    public $id, $cuenta, $clave_catastral, $nombre, $domicilio_predio, $calle, 
           $colonia, $poblacion, $domicilio_notificacion, $valor_fiscal, $tasa, 
           $inicio, $fin, $impuesto, $recargos, $rezagos, $gastos, $multas, $adeudo;

    protected $listeners = ['datosRecibidos' => 'guardarPredio'];

    protected function rules()
    {
        return [
            'cuenta' => 'required|string|unique:predios,cuenta,' . $this->id,
            'clave_catastral' => 'required|string',
            'nombre' => 'required|string',
            'domicilio_predio' => 'required|string',
            'calle' => 'required|string',
            'colonia' => 'required|string',
            'poblacion' => 'required|string',
            'domicilio_notificacion' => 'required|string',
            'valor_fiscal' => 'required|numeric|min:0',
            'tasa' => 'required|numeric|min:0|max:100',
            'inicio' => 'required|date',
            'fin' => 'required|date|after_or_equal:inicio',
            'impuesto' => 'required|numeric|min:0',
            'recargos' => 'required|numeric|min:0',
            'rezagos' => 'required|numeric|min:0',
            'gastos' => 'required|numeric|min:0',
            'multas' => 'required|numeric|min:0',
            'adeudo' => 'required|numeric|min:0',
        ];
    }

    // Función para mostrar el modal de creación o edición
    public function abrirModal($id = null)
    {
        if ($id) {
            $predio = Predio::findOrFail($id);
            $this->fill($predio->toArray());
            $this->modoEdicion = true;
        } else {
            $this->reset();
            $this->modoEdicion = false;
        }
        $this->modalVisible = true;
    }

    // Al actualizar la búsqueda, vuelve a la primera página
        public function updatedSearch()
        {
            $this->resetPage();
        }

        // Al cambiar la cantidad de elementos por página, vuelve a la primera página
        public function updatedPerPage()
        {
            $this->resetPage();
        }

    // Función para guardar el predio
    public function guardarPredio($datos)
    {
        // Validar los datos manualmente en lugar de $this->validate()
        $validados = Validator::make($datos, $this->rules())->validate();

    
        if ($this->modoEdicion) {
            Predio::where('id', $datos['id'])->update($validados);
            session()->flash('success', 'Predio actualizado correctamente.');
        } else {
            Predio::create($validados);
            session()->flash('success', 'Predio creado correctamente.');
        }
    
        $this->modalVisible = false;
    }
    // Función para eliminar un predio
    public function eliminar($id)
    {
        Predio::findOrFail($id)->delete();
        session()->flash('success', 'Predio eliminado correctamente.');
    }

    // Función de renderizado
    public function render()
    {
        $predios = Predio::query()
            ->where(function ($query) {
                // Búsqueda optimizada para buscar en cualquier campo
                $query->where('cuenta', 'like', '%'.$this->search.'%')
                      ->orWhere('clave_catastral', 'like', '%'.$this->search.'%')
                      ->orWhere('nombre', 'like', '%'.$this->search.'%')
                      ->orWhere('domicilio_predio', 'like', '%'.$this->search.'%')
                      ->orWhere('calle', 'like', '%'.$this->search.'%')
                      ->orWhere('colonia', 'like', '%'.$this->search.'%')
                      ->orWhere('poblacion', 'like', '%'.$this->search.'%')
                      ->orWhere('valor_fiscal', 'like', '%'.$this->search.'%');
            })
            ->paginate($this->perPage); // Aplicar la paginación

        return view('livewire.predios', compact('predios'));
    }
}
