<?php

namespace App\Imports;

use App\Models\PagoImport;
use Maatwebsite\Excel\Concerns\ToModel;
use Maatwebsite\Excel\Concerns\WithValidation;
use Maatwebsite\Excel\Concerns\Importable;
use Maatwebsite\Excel\Concerns\WithHeadingRow;
use Maatwebsite\Excel\Concerns\WithSkipDuplicates;

use Illuminate\Support\Facades\Auth;
use Carbon\Carbon;

class PagosImport implements ToModel, WithValidation, WithHeadingRow, WithSkipDuplicates
{
    use Importable;

    /**
     * Convierte cada fila a un modelo
     *
     * @param array $row
     * @return \Illuminate\Database\Eloquent\Model|null
     */
    public function model(array $row)
    {
        $usuario = auth()->user();
        return new PagoImport([
            'recibo' => $row['recibo'],
            'cuenta' => $row['cuenta'],
            'predial' => $row['predial'],
            'recargos' => $row['recargos'],
            'rezago' => $row['rezago'],
            'multas' => $row['multas'],
            'gastos_cobranza' => $row['gastos_cobranza'],
            'gastos_envio' => $row['gastos_envio'],
            'gastos_notificacion' => $row['gastos_notificacion'],
            // 'importe' => $row['importe'],
            'descuento' => $row['descuento'],
            // 'fecha_pago' => '01/02/2025',
            'fecha_pago' => $this->formatFecha($row['fecha_pago']),
            'estado' => 1,
            'creado_por' => $usuario->id,
        ]);
    }

    private function formatFecha($fecha)
    {
        if (!$fecha) {
            return null;
        }

        // Si es un timestamp de Excel (número), convertirlo
        if (is_numeric($fecha)) {
            return Carbon::instance(\PhpOffice\PhpSpreadsheet\Shared\Date::excelToDateTimeObject($fecha))
                ->format('Y-m-d');
        }

        return $fecha; // Si ya es una fecha válida
    }

    /**
     * Define las reglas de validación para cada columna
     *
     * @return array
     */
    public function rules(): array
    {
        return [
            'recibo' => 'required|string|max:255',
            'cuenta' => 'required|string|max:255',
            'predial' => 'required|numeric|between:0,999999999999999.99',
            'recargos' => 'required|numeric|between:0,999999999999999.99',
            'rezago' => 'required|numeric|between:0,999999999999999.99',
            'multas' => 'required|numeric|between:0,999999999999999.99',
            'gastos_cobranza' => 'required|numeric|between:0,999999999999999.99',
            'gastos_envio' => 'required|numeric|between:0,999999999999999.99',
            'gastos_notificacion' => 'required|numeric|between:0,999999999999999.99',
            // 'importe' => 'required|numeric|between:0,999999999999999.99',
            'descuento' => 'nullable|numeric|between:0,999999999999999.99',
            // 'fecha_pago' => 'required',
            'creado_por' => 'nullable|integer',
        ];
    }

    /**
     * Mensajes personalizados de error
     *
     * @return array
     */
    public function customValidationMessages()
    {
        return [
            'recibo.required' => 'El recibo es obligatorio.',
            'cuenta.required' => 'La cuenta es obligatoria.',
            'predial.required' => 'El número predial es obligatorio.',
            'recargos.required' => 'Los recargos son obligatorios y deben ser un número válido.',
            'rezago.required' => 'El rezago es obligatorio y debe ser un número válido.',
            'multas.required' => 'Las multas son obligatorias y deben ser un número válido.',
            'gastos_cobranza.required' => 'Los gastos de cobranza son obligatorios y deben ser un número válido.',
            'gastos_envio.required' => 'Los gastos de envío son obligatorios y deben ser un número válido.',
            'gastos_notificacion.required' => 'Los gastos de notificación son obligatorios y deben ser un número válido.',
            // 'importe.required' => 'El importe es obligatorio y debe ser un número válido.',
            'descuento.numeric' => 'El descuento debe ser un número válido.',
            'fecha_pago.required' => 'La fecha de pago es obligatoria.',
            'fecha_pago.date' => 'La fecha de pago debe ser una fecha válida.',
            'creado_por.integer' => 'El campo creado por debe ser un número entero.',
        ];
    }
}
