<?php

namespace App\Imports;

use App\Models\PredioImport;
use Maatwebsite\Excel\Concerns\ToModel;
use Maatwebsite\Excel\Concerns\WithValidation;
use Maatwebsite\Excel\Concerns\Importable;
use Maatwebsite\Excel\Concerns\WithHeadingRow;
use Illuminate\Support\Facades\Auth;
use Carbon\Carbon;

class PrediosImport implements ToModel, WithValidation, WithHeadingRow
{
    use Importable;

    /**
     * Convierte cada fila a un modelo
     *
     * @param array $row
     * @return \Illuminate\Database\Eloquent\Model|null
     */
    public function model(array $row)
    {
        $usuario = auth()->user();
        return new PredioImport([
            'cuenta' => $row['cuenta'],
            'clave_catastral' => $row['clave_catastral'],
            'nombre' => $row['nombre'],
            'domicilio_predio' => $row['domicilio_predio'],
            'calle' => $row['calle'],
            'colonia' => $row['colonia'],
            'poblacion' => $row['poblacion'],
            'domicilio_notificacion' => $row['domicilio_notificacion'],
            'valor_fiscal' => $row['valor_fiscal'],
            'tasa' => $row['tasa'],
            'inicio' => $this->formatFecha($row['inicio']),
            'fin' =>  $this->formatFecha($row['fin']),
            // 'inicio' => Carbon::createFromFormat('dd/mm/YYYY', $row['inicio'])->format('Y-m-d'),
            // 'fin' => Carbon::createFromFormat('dd/mm/YYYY', $row['fin'])->format('Y-m-d'),
            'impuesto' => $row['impuesto'],
            'recargos' => $row['recargos'],
            'rezagos' => $row['rezagos'],
            'gastos' => $row['gastos'],
            'multas' => $row['multas'],
            'adeudo' => $row['adeudo'],
            'historico' => 0,
            'notificador' =>1,
            'creado_por' => $usuario->id,
        ]);
    }

    private function formatFecha($fecha)
    {
        if (!$fecha) {
            return null;
        }

        // Si es un timestamp de Excel (número), convertirlo
        if (is_numeric($fecha)) {
            return Carbon::instance(\PhpOffice\PhpSpreadsheet\Shared\Date::excelToDateTimeObject($fecha))
                ->format('Y-m-d');
        }
    }

    /**
     * Define las reglas de validación para cada columna
     *
     * @return array
     */
    public function rules(): array
    {
        return [
            'cuenta' => 'required|string|max:255',
            'clave_catastral' => 'required|string|max:255',
            'nombre' => 'required|string|max:255',
            'domicilio_predio' => 'required|string|max:255',
            'calle' => 'required|string|max:255',
            'colonia' => 'required|string|max:255',
            'poblacion' => 'required|string|max:255',
            'domicilio_notificacion' => 'required|string|max:255',
            'valor_fiscal' => 'required|numeric|between:0,999999999999999.99',
            'tasa' => 'required|numeric|between:0,99.99',
            // 'inicio' => 'required|date',
            // 'fin' => 'required|date',
            'impuesto' => 'required|numeric|between:0,999999999999999.99',
            'recargos' => 'required|numeric|between:0,999999999999999.99',
            'rezagos' => 'required|numeric|between:0,999999999999999.99',
            'gastos' => 'required|numeric|between:0,999999999999999.99',
            'multas' => 'required|numeric|between:0,999999999999999.99',
            'adeudo' => 'required|numeric|between:0,999999999999999.99',
            'historico' => 'nullable|integer',
            'notificador' => 'nullable|integer',
            'creado_por' => 'nullable|integer',
        ];
    }

    /**
     * Mensajes personalizados de error
     *
     * @return array
     */
    public function customValidationMessages()
    {
        return [
            'cuenta.required' => 'La cuenta es obligatoria.',
            'clave_catastral.required' => 'La clave catastral es obligatoria en la fila :row.',
            'nombre.required' => 'El nombre es obligatorio.',
            'domicilio_predio.required' => 'El domicilio del predio es obligatorio.',
            'calle.required' => 'La calle es obligatoria.',
            'colonia.required' => 'La colonia es obligatoria.',
            'poblacion.required' => 'La población es obligatoria.',
            'domicilio_notificacion.required' => 'El domicilio de notificación es obligatorio.',
            'valor_fiscal.required' => 'El valor fiscal es obligatorio y debe ser un número válido.',
            'tasa.required' => 'La tasa es obligatoria y debe ser un número válido.',
            'inicio.required' => 'La fecha de inicio es obligatoria y debe ser válida.',
            'fin.required' => 'La fecha de fin es obligatoria y debe ser válida.',
            'impuesto.required' => 'El impuesto es obligatorio y debe ser un número válido.',
            'recargos.required' => 'Los recargos son obligatorios y deben ser un número válido.',
            'rezagos.required' => 'Los rezagos son obligatorios y deben ser un número válido.',
            'gastos.required' => 'Los gastos son obligatorios y deben ser un número válido.',
            'multas.required' => 'Las multas son obligatorias y deben ser un número válido.',
            'adeudo.required' => 'El adeudo es obligatorio y debe ser un número válido.',
            'historico.integer' => 'El histórico debe ser un número entero.',
            'notificador.integer' => 'El notificador debe ser un número entero.',
            'creado_por.integer' => 'El creador debe ser un número entero.',
        ];
    }
}

